\ create combined versions of possibly STATE-smart words

\ This file is in the public domain. NO WARRANTY.

s" combined.fs" included

: compile-[ ( -- )
    postpone [ ;

: compile-only-error ( -- )
    -14 throw ;

: repair ( "name" -- )
    \ defines a word "name" whose compilation semantics is the
    \ compilation semantics of the old "name" in compile state,
    \ and whose interpretation semantics is the interpretation semantics
    \ of the old "name" in interpret state.
    \ Summary: Converts a STATE-smart word into a combined word
    \          does not hurt STATE-dumb compiling words
    \          and tries not to hurt other systems that do it right
    >in @ >r
    \ build a word for the interpretation semantics in interpret STATE
    bl word find 0= if \ try to access the interpretation semantics
	drop ['] compile-only-error \ this word has none on this system
    then
    r@ >in ! >r
    :noname postpone state postpone @ postpone if
        postpone compile-[ r@ compile, postpone ]
    postpone else
	r> compile,
    postpone then postpone ;
    \ build a word for the compilation semantics in compile STATE
    :noname postpone state postpone @ postpone if
	postpone postpone r@ >in ! 
    postpone else
        postpone ] postpone postpone r@ >in ! postpone compile-[
    postpone then postpone ;
    interpret/compile:
    r> drop ;

\ These are all words that have an interpretation semantics entry and
\ a compilation semantics entry

\ you can take out everything that is not implemented STATE-smartly on
\ your system

repair +loop
repair ."
\ repair ; \ see below
repair abort"
repair begin
repair do
repair does>
repair else
repair if
\ repair literal \ unnecessary, because LITERAL is defined in combined.fs
repair loop
\ repair postpone \ unnecessary, because POSTPONE is defined in combined.fs
repair recurse
repair repeat
repair s"
repair then
repair until
repair while
\ repair ['] \ unnecessary, because ['] is defined in combined.fs
repair [char]
repair ?do
repair again
repair c"
repair case
repair endcase
repair endof
repair of
\ repair to \ cannot be POSTPONEd, so we cannot repair it (but that's ok)
repair 2literal
repair fliteral
repair to
repair locals|
\ repair ;code \ see below
repair ahead
repair sliteral

\ ; and ;code change STATE, so REPAIR does not work for them.
\ However, we can repair them in the following STATE-dumb way:

: ; ] postpone ; ; immediate
\ : ;code ] postpone ;code ; immediate \ not present in Gforth

\ Immediate words with undefined interpretation semantics: [
\ repaired in a STATE-dumb way
: [ ] postpone [ ; immediate

\ words with execution semantics with undefined interpretation semantics:
\ (local) cs-pick cs-roll >r exit i j leave r> r@ unloop 2>r 2r> 2r@ compile, 

\ I did not bother to repair these words, because I don't think
\ anybody would make them STATE-smart, and the present version of
\ repair does not do what you expect for these words (in particular
\ when they are [compile]d).


